//imports
import java.util.*;
import javax.swing.table.*;
import javax.swing.*;
import javax.swing.event.*; 
import javax.swing.event.TableModelEvent.*;
import java.awt.*;
import java.awt.event.ActionEvent;

/**
 * Classe ListeUtilisateurs
 * Liste d'objet Utilisateur
 * 
 * @author Montet/Villemaine 
 * @version 0.1
 */
class ListeUtilisateurs implements TableModel
{
    //variables
    private ArrayList listeUtilisateurs; //liste des Utilisateurs
    private ListePanneauUtilisateurs UI; //Liste des panneaux d'affichage
    private ArrayList listeListeners;    //liste des listeners pour la JTable (renderer)
    private JFrame fenAff;               //fenetre ou est affiche l'UI;
    
    /**
     * Constructeur sans paramtres.
     */
    public ListeUtilisateurs()
    {
        UI=null;
        listeUtilisateurs = new ArrayList();
        listeListeners = new ArrayList();
        fenAff = null;
    }

    /**
     * Ajout d'un nouvel utilisateur dans la liste.
     * @param util l'utilisateur dsir
     */
    public void ajouter(Utilisateur util)
    {
        //si il n'existe pas un utilisateur avec le mme login   
        if (listeUtilisateurs.indexOf(util)==-1)
        {
            //on l'ajoute dans la lite   
            listeUtilisateurs.add(util);
            //si l'affichage est activ   
            if (UI!=null)
            {
                //on a l'ajoute  l'affichage   
                UI.ajouterUtilisateur(util);
                //on prvient les listeners pour la JTable
                this.fireTableRowsInserted(0,this.getLength());
            }
        }
        else 
        {
           JOptionPane.showMessageDialog(fenAff,"Un utilisateur du mme nom existe dj !");              
       }

    }

    /**
     * Ajout d'un nouvel utilisateur dans la liste
     * en passant les paramtres de l'utilisateur.
     * @param login le login de l'utilisateur dsir
     * @param password le password de l'utilisateur dsir
     * @param privilege le privilge de l'utilisateur dsir
     */
    public void ajouter(String login, String password, int privilege)
    {
        //cration d'un utilisateur   
        Utilisateur  util2 = new Utilisateur(login,password,privilege);
        //appel de ajouter() qui prends un Utilisateur en paramtre
        ajouter(util2);
    } 
    
    /**
     * Suppression d'un utilisateur dans la liste
     * @param login le login de l'utilisateur que l'on veut supprimer.
     */    
    public void effacer(String login)
    {
        //cration d'un Utilisateur avec le mme login   
        Utilisateur  util2 = new Utilisateur(login,"",0);
        //on enlve l'utilisateur de la liste qui possde le mme
        //login (nb: le login est unique)
        listeUtilisateurs.remove(listeUtilisateurs.indexOf(util2));
    }

    /**
     * Accesseur de la longueur de la liste d'utilisateur.
     * @return la taille de la liste.
     */      
    public int getLength()
    {
         return listeUtilisateurs.size();
    }

    /**
     * Accesseur du login d'un utilisateur de la liste.
     * @param i l'indice de l'utilisateur dans la liste.
     * @return le login de l'utilisateur au rang spcifi.
     */      
    public String getLogin(int i)
    {
         return ((Utilisateur)listeUtilisateurs.get(i)).getLogin();
    }

    /**
     * Accesseur du password d'un utilisateur de la liste.
     * @param i l'indice de l'utilisateur dans la liste.
     * @return le password de l'utilisateur au rang spcifi.
     */    
    public String getPassword(int i)
    {
         return ((Utilisateur)listeUtilisateurs.get(i)).getPassword();
    }

    /**
     * Accesseur du privilge d'un utilisateur de la liste.
     * @param i l'indice de l'utilisateur dans la liste.
     * @return le privilge de l'utilisateur au rang spcifi.
     */      
    public int getPrivilege(int i)
    {
         return ((Utilisateur)listeUtilisateurs.get(i)).getPrivilege();
    }

    /**
     * Accesseur de l'indice d'un utilisateur de la liste a partir de son login.
     * @param login le login de l'utilisateur dans la liste.
     * @return l'indice de l'utilisateur au login spcifi.
     */      
    public int getIndice(String login)
    {
         Utilisateur  util2 = new Utilisateur(login,"",0);
         return listeUtilisateurs.indexOf((Utilisateur)util2);
    }

    /**
     * Redfinition de la mthode toString pour l'affichage
     * de la liste
     * @return un chaine de charactres formatte contenant les informations de la liste
     */    
    public String toString()
    {
       String ch="";
       for (int i=0;i<listeUtilisateurs.size();i++)
        {
            //utilisation du toString de la classe Utilisateur
            ch = ch+(Utilisateur)listeUtilisateurs.get(i)+"\n";
        }
        return ch;
    }

    /**
     * Accesseur de l'utilisateur de la liste a partir de son indice.
     * @param indice l'indice de l'utilisateur dans la liste.
     * @return l'utilisateur au rang spcifi.
     */      
    public Utilisateur getUtilisateur(int indice)
    {
       return (Utilisateur)listeUtilisateurs.get(indice);
    }
    
    /**
     * Modificateur de l'afficheur de la liste
     * @param liste la liste de panneau utilisateurs utilise pour l'affichage
     * @param fen la fentre ou est affiche l'UI
     */     
    public void setUI(ListePanneauUtilisateurs liste, JFrame fen)
    {
        UI=liste;
        fenAff=fen;
    }

    /**
     * Implmentation de mthode ncessaire pour TableModel.
     * Retourne le nombre de colonnes de la table.
     * @return le nombre de colonnes.
     */    
    public int getColumnCount()
    {
       return 3;
    }

    /**
     * Implmentation de mthode ncessaire pour TableModel.
     * Retourne l'objet au rang (parm1,parm2) de la table.
     * @param parm1 la ligne voulue dans la table
     * @param parm2 la colonne voulue dans la table.
     * @return l'objet contenu dans la cellule dsigne.
     */    
    public Object getValueAt(int parm1, int parm2)
    {
       //on switch sur la colonne
       switch (parm2)
       {
            case 0: //on retourn le login 
                    return this.getLogin(parm1);  
            case 1: //on reoturne le password 
                    return this.getPassword(parm1);  
            case 2: //on retourne le privilge 
                    return new Integer(this.getPrivilege(parm1));
            default : return null;
       }
    }

    /**
     * Implmentation de mthode ncessaire pour TableModel.
     * Retourne le nombre de ligne de la table.
     */    
    public int getRowCount()
    {
       return this.getLength();
    }

    /**
     * Implmentation de mthode ncessaire pour TableModel.
     * Retourne le nom de la colonne de la table.
     * @param col l'indice de la colonne dont on veut avoir le nom.
     * @return une chaine de charactres contenant le nom de la colonne.
     */    
    public String getColumnName(int col)
    {
       //on switch sur le numro de la colonne et
       //on retourne le nom de celle-ci
       switch (col)
       {
            case 0: return "Nom";  
            case 1: return "MDP";  
            case 2: return "Privilge"; 
            default : return null;
        }
    }

    /**
     * Implmentation de mthode ncessaire pour TableModel.
     * Retourne si la cellule est ditable.
     * Retourne toujours vrai.
     * @param row la ligne de la cellule
     * @param col la colonne de la cellule
     * @return un boolen toujours vrai
     */       
    public boolean isCellEditable(int row, int col)
    {
       
       return true;
    }

    /**
     * Implmentation de mthode ncessaire pour TableModel.
     * Retourne la classe des objets contenus dans la colonne spcifie.
     * @param col le numro de colonne.
     * @return la classe de l'objet.
     */           
    public Class getColumnClass(int col)
    {
        return getValueAt(0, col).getClass();
    } 
    
    /**
     * Implmentation de mthode ncessaire pour TableModel.
     * Ajoute un listener de table.
     * @param l un TableModelListener.
     */    
    public void addTableModelListener(TableModelListener l) 
    {
        listeListeners.add(l);
    }
       
    /**
     * Implmentation de mthode ncessaire pour TableModel.
     * Supprime un listener de table.
     * @param l un TableModelListener.
     */            
    public void removeTableModelListener(TableModelListener l) 
    {
        listeListeners.remove(l);      
    }
    
    /**
     * Implmentation de mthode ncessaire pour TableModel.
     * Modificateur d'une cellule.
     * @param aValue l'objet qui doit tre mis dans la cellule
     * @param rowIndex la ligne de la cellule
     * @param columnIndex la colonne de la cellule.
     */           
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) 
    {
         //cration d'un couteur de menu
         MenuEcouteur ME = new MenuEcouteur(fenAff);
         switch (columnIndex)
         {
              case 0: this.getUtilisateur(rowIndex).setLogin(aValue.toString());
                      //fenAff.getContentPane().getComponent(0).repaint();
                      break;
              case 1: this.getUtilisateur(rowIndex).setPassword(aValue.toString());
                      break;
              case 2: this.getUtilisateur(rowIndex).setPrivilege((int)(Integer.parseInt(aValue.toString())));
                      break;
         }
         //envoi d'une action de clic fictif sur le menu "Voir la liste"   
         //ME.actionPerformed(new ActionEvent(new Object(),0, "Voir la liste"));
    }
    
    /**
     * Implmentation de mthode ncessaire pour TableModel.
     * Fonction qui permet d'alerter lors de l'ajout/suppression/modification 
     * d'une cellule dans la table.
     * @param firstRow premire ligne concerne
     * @param lastRow dernire ligne concerne.
     */        
    public void fireTableRowsInserted(int firstRow, int lastRow)
    {
       for (int i=0;i<listeListeners.size();i++)
       {
             ((TableModelListener)listeListeners.get(i)).tableChanged(new TableModelEvent(this, firstRow, lastRow, TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
        }        
    }

           

          


}
